import 'dart:collection';

import 'package:cramr_ui_lib/ads/safe_banner_ad_widget.dart';
import 'package:cramr_ui_lib/model/verb.dart';
import 'package:cramr_ui_lib/repo/verb_repository.dart';
import 'package:cramr_ui_lib/routes/verb_studymode_selection_route.dart';
import 'package:cramr_ui_lib/settings/settings_controller.dart';
import 'package:cramr_ui_lib/settings/settings_view.dart';
import 'package:cramr_ui_lib/util/practice_session_util.dart';
import 'package:cramr_ui_lib/util/text_size_limiter.dart';
import 'package:cramr_ui_lib/widgets/delayed_widget_builder.dart';
import 'package:flutter/material.dart';

/// The Route that allows you to select a verb for evaluation
class MultiConjugationVerbSelectionRoute extends StatefulWidget {
  static const routeName = '/multiConjugationVerbSelectionRoute';

  final SettingsController _settingsController;
  final SafeBannerAdWidget safeBannerAdWidget;

  const MultiConjugationVerbSelectionRoute(this._settingsController,
      {Key? key, required this.safeBannerAdWidget})
      : super(key: key);

  @override
  State<MultiConjugationVerbSelectionRoute> createState() =>
      MultiConjugationVerbSelectionRouteState();

  static Key verbIconKey(String infinitive, String mode, String tense) {
    return Key('startStudy_${infinitive}_${mode}_$tense');
  }
}

class MultiConjugationVerbSelectionRouteState
    extends State<MultiConjugationVerbSelectionRoute> {
  final HashMap<String, Verb> verbs = VerbRepository.instance().verbs;

  List<String>? filteredVerbs = [];

  @override
  Widget build(BuildContext context) {
    return AnimatedBuilder(
        animation: widget._settingsController,
        builder: (BuildContext context, Widget? child) {
          return MaterialApp(
            debugShowCheckedModeBanner: false,
            themeMode: widget._settingsController.themeMode,
            theme: ThemeData(),
            darkTheme: ThemeData.dark(),
            home: SafeArea(
              child: Scaffold(
                appBar: AppBar(
                  leading: IconButton(
                      key: const Key('BackKey'),
                      onPressed: () {
                        Navigator.pop(context);
                      },
                      icon: const Icon(Icons.arrow_back)),
                  actions: [_searchIcon(context), _settingsIcon(context)],
                ),
                body: Column(
                  children: [
                    Expanded(
                      child: Center(
                        key: const Key('verbSelectionCenter'),
                        child: _verbListView(context),
                      ),
                    ),
                    Container(
                        height: 100.0,
                        alignment: Alignment.center,
                        child: widget.safeBannerAdWidget)
                  ],
                ),
                // This trailing comma makes auto-formatting nicer for build methods.
              ),
            ),
          );
        });
  }

  Widget _verbListView(BuildContext context) {
    return ListView(
      key: const Key('verbListView'),
      scrollDirection: Axis.vertical,
      controller: ScrollController(),
      physics: const ScrollPhysics(
        parent: AlwaysScrollableScrollPhysics(),
      ),
      children: _repoButtons(context),
    );
  }

  IconButton _searchIcon(BuildContext context) {
    return IconButton(
      key: const Key("searchVerbButton"),
      onPressed: () async {
        String? filteredVerb = await showSearch(
            context: context,
            delegate: VerbSearchDelegate(),
            useRootNavigator: true);
        setState(() {
          filteredVerbs!.clear();
          filteredVerbs!.add(filteredVerb!);
        });
      },
      icon: const Icon(Icons.search),
    );
  }

  IconButton _settingsIcon(BuildContext context) {
    return IconButton(
        key: const Key('settingsButton'),
        icon: const Icon(Icons.settings),
        onPressed: () {
          Navigator.pushNamed(context, SettingsView.routeName);
        });
  }

  List<Widget> _repoButtons(BuildContext context) {
    List<Widget> result = [];

    for (var key in filteredVerbs!) {
      if (verbs.containsKey(key)) {
        Verb? value = verbs[key];
        result.add(SizedBox(
          child: Card(
            key: Key('verbCard_$value'),
            margin: const EdgeInsets.fromLTRB(30, 15, 30, 15),
            elevation: 10.0,
            child: _verbButton(context, value!),
          ),
        ));
      }
    }

    return result;
  }

  Widget _verbButton(BuildContext context, Verb verb) {
    return Padding(
      padding: const EdgeInsets.all(8.0),
      child: Column(
        key: Key("vervoeg ${verb.infinitive}"),
        children: [
          SizedBox(
            key: UniqueKey(),
            height: 40,
            child: Text(
              verb.infinitive,
              key: UniqueKey(),
              style: Theme.of(context).textTheme.headline5,
              textScaleFactor: TextSizeLimiter.textScaleFactor(context),
            ),
          ),
          SizedBox(
            key: UniqueKey(),
            height: 40,
            child: Text(
              verb.translation,
              key: UniqueKey(),
              style: Theme.of(context).textTheme.bodyText2,
              textScaleFactor: TextSizeLimiter.textScaleFactor(context),
            ),
          ),
          verb.hasSimplePastIndicative()
              ? Card(
                  elevation: 5.0,
                  child: Column(children: _conjugationCards(context, verb)),
                )
              : _verbIcon(context, verb, null, null),
        ],
      ),
    );
  }

  List<Widget> _conjugationCards(BuildContext context, Verb verb) {
    List<Widget> result = [];
    verb.indicativeConjugations.forEach((key, value) {
      result.add(_conjugationCard(context, value.mode, value.tense, verb));
    });
    return result;
  }

  Widget _conjugationCard(
      BuildContext context, String mode, String tense, Verb verb) {
    //TODO evaluate whether this can be replaced with VerbConjugationSelector
    return Card(
      elevation: 5.0,
      child: ListTile(
        title: Text(
          '$mode $tense',
          style: Theme.of(context).textTheme.caption,
        ),
        trailing: _verbIcon(context, verb, mode, tense),
      ),
    );
  }

  List<String> indicativeForVerbIcon(Verb verb, String? mode, String? tense) {
    List<String> indicative = verb.simplePresentIndicative;
    if (null != tense) {
      if ('présent' == tense && 'indicatif' == mode) {
        indicative = verb.simplePresentIndicative;
      } else {
        indicative = verb.indicativeConjugations['${mode}_$tense']!.conjugation;
      }
    }
    return indicative;
  }

  Key _iconKey(Verb verb, String? mode, String? tense) {
    Key iconKey = Key('startStudy_${verb.infinitive}');
    if (null != tense) {
      if ('présent' == tense && 'indicatif' == mode) {
        iconKey = Key('startStudy_${verb.infinitive}_indicatif_présent');
      } else {
        iconKey = MultiConjugationVerbSelectionRoute.verbIconKey(
            verb.infinitive, mode!, tense);
      }
    }
    return iconKey;
  }

  Widget _verbIcon(
      BuildContext context, Verb verb, String? mode, String? tense) {
    Key iconKey = _iconKey(verb, mode, tense);
    return DelayedWidgetBuilder(
      delay: const Duration(milliseconds: 500),
      child: InkWell(
        key: UniqueKey(),
        child: CircleAvatar(
            key: UniqueKey(), child: Icon(Icons.play_arrow, key: iconKey)),
        onTap: () {
          List<String> indicative = indicativeForVerbIcon(verb, mode, tense);
          prepareModelForSession(context, indicative, verb, mode, tense);
          goToStudyModeSelection(context);
        },
      ),
    );
  }

  static void goToStudyModeSelection(BuildContext context) {
    Navigator.pushNamed(context, VerbStudymodeSelectionRoute.routeName);
  }

  void prepareModelForSession(BuildContext context, List<String> indicative,
      Verb verb, String? mode, String? tense) {
    PracticeSessionUtil.initialiseVerbTestModel(
        context,
        widget._settingsController.cardinalitySelection,
        indicative,
        verb,
        mode!,
        tense!);
  }

  @override
  void initState() {
    super.initState();
    verbs.forEach((key, value) {
      filteredVerbs!.add(key);
    });
  }
}

class VerbSearchDelegate extends SearchDelegate<String> {
  List<String> allVerbs = [];
  List<String> suggestedVerbs = [];

  VerbSearchDelegate() {
    var verbs = VerbRepository.instance().verbs;
    for (var element in verbs.keys) {
      allVerbs.add(element);
    }
  }

  @override
  List<Widget>? buildActions(BuildContext context) {
    List<Widget> actions = [];
    return actions;
  }

  @override
  Widget? buildLeading(BuildContext context) {
    return null;
  }

  @override
  Widget buildResults(BuildContext context) {
    for (var element in allVerbs) {
      if (element.toLowerCase().contains(query.toLowerCase())) {
        suggestedVerbs.add(element);
      }
    }

    return ListView.builder(
        itemCount: suggestedVerbs.length,
        itemBuilder: (BuildContext context, int index) {
          var result = suggestedVerbs[index];
          return ListTile(
            title: Text(
              result,
              key: Key("searchResult_$result"),
              textScaleFactor: TextSizeLimiter.textScaleFactor(context),
            ),
            onTap: () {
              close(context, result);
            },
          );
        });
  }

  @override
  Widget buildSuggestions(BuildContext context) {
    suggestedVerbs.clear();
    for (var element in allVerbs) {
      if (element.toLowerCase().contains(query.toLowerCase())) {
        suggestedVerbs.add(element);
      }
    }

    return ListView.builder(
        itemCount: suggestedVerbs.length,
        itemBuilder: (BuildContext context, int index) {
          var result = suggestedVerbs[index];
          return ListTile(
            title: Text(
              result,
              key: Key("suggestedResult_$result"),
              textScaleFactor: TextSizeLimiter.textScaleFactor(context),
            ),
            onTap: () {
              close(context, result);
            },
          );
        });
  }
}

class VerbPanelItem {
  late final String mode;
  late final String tense;
  bool isExpanded;

  VerbPanelItem(
      {required this.mode, required this.tense, this.isExpanded = false});
}
