import 'dart:io';
import 'dart:ui';

import 'package:flutter/foundation.dart';
import 'package:flutter_local_notifications/flutter_local_notifications.dart';
import 'package:flutter_native_timezone/flutter_native_timezone.dart';
import 'package:timezone/data/latest_all.dart' as tz;
import 'package:timezone/timezone.dart' as tz;

class NotificationService {
  NotificationService();

  final _localNotifications = FlutterLocalNotificationsPlugin();

  Future<void> initializePlatformNotifications(
      AndroidInitializationSettings initializationSettingsAndroid) async {
    tz.initializeTimeZones();
    _isAndroidPermissionGranted();
    final String timeZone = await FlutterNativeTimezone.getLocalTimezone();
    tz.setLocalLocation(tz.getLocation(timeZone));

    final DarwinInitializationSettings initializationSettingsIOS =
        DarwinInitializationSettings(
            requestSoundPermission: true,
            requestBadgePermission: true,
            requestAlertPermission: true,
            onDidReceiveLocalNotification: onDidReceiveLocalNotification);

    final InitializationSettings initializationSettings =
        InitializationSettings(
      android: initializationSettingsAndroid,
      iOS: initializationSettingsIOS,
    );

    await _localNotifications.initialize(initializationSettings);
  }

  void onDidReceiveLocalNotification(
      int id, String? title, String? body, String? payload) {
    print('id $id');
  }

  Future<NotificationDetails> _notificationDetails() async {
    AndroidNotificationDetails androidPlatformChannelSpecifics =
        const AndroidNotificationDetails(
      'channel id',
      'channel name',
      groupKey: 'be.teamwabisabi.verbomatique_notifications',
      channelDescription: 'verbomatique_notifications',
      importance: Importance.defaultImportance,
      priority: Priority.defaultPriority,
      playSound: false,
      ticker: 'ticker',
      color: Color(0xff2196f3),
    );

    DarwinNotificationDetails iosNotificationDetails =
        const DarwinNotificationDetails(
      threadIdentifier: "thread1",
    );

    NotificationDetails platformChannelSpecifics = NotificationDetails(
        android: androidPlatformChannelSpecifics, iOS: iosNotificationDetails);

    return platformChannelSpecifics;
  }

  Future<void> scheduleNotification(
      {required int hour,
      required int minute,
      required int id,
      required String title,
      required String body,
      required String payload}) async {
    final platformChannelSpecifics = await _notificationDetails();
    tz.TZDateTime scheduledDate = _nextInstanceOfTime(hour, minute);
    if (kDebugMode) {
      print(scheduledDate);
    }
    await _localNotifications.zonedSchedule(
        id, title, body, scheduledDate, platformChannelSpecifics,
        uiLocalNotificationDateInterpretation:
            UILocalNotificationDateInterpretation.absoluteTime,
        androidAllowWhileIdle: true);
    List<ActiveNotification> activeNotifications =
        await _localNotifications.getActiveNotifications();
    if (kDebugMode) {
      print('active notifications:');
      for (ActiveNotification notification in activeNotifications) {
        print(notification.body);
      }
    }
    List<PendingNotificationRequest> requests =
        await _localNotifications.pendingNotificationRequests();
    if (kDebugMode) {
      print('pending requests:');
    }
    for (PendingNotificationRequest request in requests) {
      if (kDebugMode) {
        print(request.title);
      }
    }
  }

  Future<void> cancelNotifications() async {
    await _localNotifications.cancelAll();
  }

  tz.TZDateTime _nextInstanceOfTime(int hour, int minute) {
    final tz.TZDateTime now = tz.TZDateTime.now(tz.local);

    tz.TZDateTime scheduledDate =
        tz.TZDateTime(tz.local, now.year, now.month, now.day, hour, minute);
    if (scheduledDate.isBefore(now)) {
      scheduledDate = scheduledDate.add(const Duration(days: 1));
    }
    if (kDebugMode) {
      print('_nextInstanceOfTime');
      print(scheduledDate);
      print('now');
      print(now);
      print('tz.local.currentTimeZone');
      print(tz.local.currentTimeZone.abbreviation);
    }
    return scheduledDate;
  }

  Future<void> showLocalNotificationAfterSeconds({
    required int seconds,
    required int id,
    required String title,
    required String body,
    required String payload,
  }) async {
    final platformChannelSpecifics = await _notificationDetails();
    await _localNotifications.zonedSchedule(
        id,
        title,
        body,
        tz.TZDateTime.now(tz.local).add(Duration(seconds: seconds)),
        platformChannelSpecifics,
        uiLocalNotificationDateInterpretation:
            UILocalNotificationDateInterpretation.absoluteTime,
        androidAllowWhileIdle: true);
  }

  Future<bool> _isAndroidPermissionGranted() async {
    bool granted = false;
    Future<bool> result = Future(() async {
      if (Platform.isAndroid) {
        granted = await _localNotifications
                .resolvePlatformSpecificImplementation<
                    AndroidFlutterLocalNotificationsPlugin>()
                ?.areNotificationsEnabled() ??
            false;
      }
      return granted;
    });
    return result;
  }
}
