import 'dart:convert';

import 'package:cramr_ui_lib/model/cardinality_selection.dart';
import 'package:cramr_ui_lib/model/streak_engine.dart';
import 'package:cramr_ui_lib/settings/notification_service.dart';
import 'package:flutter/foundation.dart';
import 'package:flutter/material.dart';
import 'package:shared_preferences/shared_preferences.dart';

import 'settings_service.dart';

/// A class that many Widgets can interact with to read user settings, update
/// user settings, or listen to user settings changes.
///
/// Controllers glue Data Services to Flutter Widgets. The SettingsController
/// uses the SettingsService to store and retrieve user settings.
class SettingsController with ChangeNotifier {
  SettingsController(this._settingsService, this._themeMode,
      this._sharedPreferences, this._notificationService);

  // Make SettingsService a private variable so it is not used directly.
  final SettingsService _settingsService;

  // // Make NotificationService a private variable so it is not used directly.
  final NotificationService _notificationService;

  // Make SharedPreferences private so it is not used directly
  final SharedPreferences _sharedPreferences;

  // Make ThemeMode a private variable so it is not updated directly without
  // also persisting the changes with the SettingsService.
  ThemeMode _themeMode;

  // Allow Widgets to read the user's preferred ThemeMode.
  ThemeMode get themeMode => _themeMode;

  // Make CardinalitySelection a private variable so that it is not updated
  // directly without persisting changes with the SettingsService
  late final CardinalitySelection _cardinalitySelection;

  // Allow Widgets to read the user's preferred CardinalitySelection
  CardinalitySelection get cardinalitySelection => _cardinalitySelection;

  // Make StreakEngine a private variable so that it is not updated
  // directly without persisting changes with the SettingsService
  late StreakEngine _streakEngine;

  // Allow Widgets to read the user's StreakEngine
  StreakEngine get streakEngine => _streakEngine;

  /// Load the user's settings from the SettingsService. It may load from a
  /// local database or the internet. The controller only knows it can load the
  /// settings from the service.
  Future<void> loadSettings() async {
    _themeMode = _settingsService.themeMode();

    // _cardinalitySelection = await _settingsService.cardinalitySelection();
    _cardinalitySelection = CardinalitySelection(
        _sharedPreferences.getBool("cardinality.singularEnabled") ?? true,
        _sharedPreferences.getBool("cardinality.pluralEnabled") ?? true);

    Object? engineVar = _sharedPreferences.get('streakEngine');
    if (engineVar == null) {
      _streakEngine = StreakEngine();
    } else {
      if (engineVar is String) {
        Map<String, dynamic> engineMap = jsonDecode(engineVar);
        _streakEngine = StreakEngine.fromJson(engineMap);
      } else {
        if (kDebugMode) {
          print(engineVar.runtimeType.toString());
        }
      }
    }

    // Important! Inform listeners a change has occurred.
    notifyListeners();
  }

  /// Update and persist the cardinality settings based on the user's selection.
  Future<void> updateCardinalitySelection() async {
    await _settingsService.updateCardinalitySelection(_cardinalitySelection);
  }

  /// Update and persist the ThemeMode based on the user's selection.
  Future<void> updateThemeMode(ThemeMode? newThemeMode) async {
    if (newThemeMode == null) return;

    // Dot not perform any work if new and old ThemeMode are identical
    //if (newThemeMode == _themeMode) return;

    // Otherwise, store the new theme mode in memory
    _themeMode = newThemeMode;

    // Important! Inform listeners a change has occurred.
    notifyListeners();

    // Persist the changes to a local database or the internet using the
    // SettingService.
    await _settingsService.updateThemeMode(newThemeMode);
  }

  /// Update and persist the StreakEngine based on the user's actions.
  Future<void> updateStreakEngine() async {
    // Inform listeners
    //notifyListeners();

    // Persist changes
    await _settingsService.updateStreakEngine(_streakEngine);
  }

  Future<void> updateNotificationEnabled(bool value) async {
    if (kDebugMode) {
      print('controller.updateNotificationEnabled');
    }
    await _settingsService.updateNotificationEnabled(value);
  }

  // Future<void> cancelNotifications() async {
  //   await _notificationService.cancelNotifications();
  // }
  //
  // Future<void> updateNotificationTime(TimeOfDay value) async {
  //   if (kDebugMode) {
  //     print('controller.updateNotificationTime');
  //   }
  //   await _notificationService.scheduleDailyHHMMNotification(
  //       value.hour, value.minute);
  //   await _settingsService.updateNotificationTime(value);
  // }

  bool isNotificationEnabled() {
    if (kDebugMode) {
      print('controller.isNotificationEnabled');
    }
    return _settingsService.isNotificationEnabled();
  }

  TimeOfDay notificationTime() {
    if (kDebugMode) {
      print('controller.notificationTime');
    }
    return _settingsService.notificationTime();
  }

  Future<void> scheduleNotification(
      {required int hour,
      required int minute,
      required String title,
      required String body,
      required String payload}) async {
    if (kDebugMode) {
      print(
          'SettingsController.scheduleNotification before updateNotificationTime');
    }
    await _settingsService
        .updateNotificationTime(TimeOfDay(hour: hour, minute: minute));
    if (kDebugMode) {
      print(
          'SettingsController.scheduleNotification before scheduleNotification');
    }
    await _notificationService.scheduleNotification(
        hour: hour,
        minute: minute,
        id: 0,
        title: title,
        body: body,
        payload: payload);
  }

  Future<void> cancelNotifications() async {
    await _notificationService.cancelNotifications();
  }

  Future<void> showLocalNotificationAfterSeconds({
    required int seconds,
    required int id,
    required String title,
    required String body,
    required String payload,
  }) async {
    await _notificationService.showLocalNotificationAfterSeconds(
        seconds: seconds, id: id, title: title, body: body, payload: payload);
  }
}
