import 'dart:convert';
import 'package:cramr_ui_lib/model/cardinality_selection.dart';
import 'package:cramr_ui_lib/model/streak_engine.dart';
import 'package:flutter/foundation.dart';
import 'package:flutter/material.dart';
import 'package:shared_preferences/shared_preferences.dart';

/// A service that stores and retrieves user settings.
///
/// By default, this class does not persist user settings. If you'd like to
/// persist the user settings locally, use the shared_preferences package. If
/// you'd like to store settings on a web server, use the http package.
class SettingsService {
  final SharedPreferences _prefs;

  SettingsService(this._prefs);

  /// Loads the User's preferred ThemeMode from local or remote storage.
  ThemeMode themeMode() {
    var themeModeIndex = _prefs.getInt("themeMode") ?? 0;
    ThemeMode result = ThemeMode.system;
    if (themeModeIndex == 1) {
      result = ThemeMode.light;
    } else if (themeModeIndex == 2) {
      result = ThemeMode.dark;
    }
    return result;
  }

  /// Persists the user's preferred ThemeMode to local or remote storage.
  Future<void> updateThemeMode(ThemeMode theme) async {
    // Use the shared_preferences package to persist settings locally or the
    // http package to persist settings over the network.
    _prefs.setInt("themeMode", theme.index);
  }

  Future<void> updateCardinalitySelection(
      CardinalitySelection selection) async {
    // Use the shared_preferences package to persist settings locally or the
    // http package to persist settings over the network.
    _prefs.setBool("cardinality.singularEnabled", selection.singularEnabled);
    _prefs.setBool("cardinality.pluralEnabled", selection.pluralEnabled);
  }

  Future<CardinalitySelection> cardinalitySelection() async {
    Future<CardinalitySelection> future = Future<CardinalitySelection>(() {
      bool? singular = _prefs.getBool("cardinality.singularEnabled");
      bool? plural = _prefs.getBool("cardinality.pluralEnabled");
      CardinalitySelection selection =
          CardinalitySelection(singular ?? true, plural ?? true);
      return selection;
    });
    return future;
  }

  /// Loads the Streak Engine
  StreakEngine streakEngine() {
    StreakEngine streakEngine = StreakEngine();
    String? jsonString = _prefs.getString('streakEngine');
    if (jsonString != null) {
      streakEngine = jsonDecode(jsonString) as StreakEngine;
    }
    return streakEngine;
  }

  /// Updates the Streak Engine
  Future<void> updateStreakEngine(StreakEngine streakEngine) async {
    _prefs.setString('streakEngine', jsonEncode(streakEngine));
  }

  Future<void> updateNotificationEnabled(bool value) async {
    if (kDebugMode) {
      print('settingsService.updateNotificationEnabled value = $value');
    }
    _prefs.setBool('notification.enabled', value);
  }

  Future<void> updateNotificationTime(TimeOfDay value) async {
    if (kDebugMode) {
      print('settingsService.updateNotificationTime $value');
    }
    _prefs.setInt('notification.hourofday', value.hour);
    _prefs.setInt('notification.minuteofday', value.minute);
  }

  bool isNotificationEnabled() {
    bool? isEnabled = _prefs.getBool('notification.enabled');
    if (null != isEnabled) {
      if (kDebugMode) {
        print('isEnabled = $isEnabled');
      }
      return isEnabled;
    }
    return true;
  }

  TimeOfDay notificationTime() {
    int? hourOfDay = _prefs.getInt('notification.hourofday');
    int? minuteOfDay = _prefs.getInt('notification.minuteofday');
    int resultHour = hourOfDay ?? 7;
    int resultMinute = minuteOfDay ?? 30;
    if (kDebugMode) {
      print('resultHour = $resultHour - resultMinute = $resultMinute');
    }
    return TimeOfDay(hour: resultHour, minute: resultMinute);
  }
}
