import 'package:cramr_ui_lib/widgets/pronoun_field.dart';
import 'package:flutter/material.dart';

/// This widget checks whether the typed word corresponds to the expected one.
/// Initially, the checkmark is grey. As long as the user types the right
/// characters, the checkmark remains orange. When wrong, the checkmark becomes
/// red. Finally, when the word is completely correct, the checkmark becomes
/// green.
class TextChecker extends StatefulWidget {
  const TextChecker(
      {Key? key,
      required this.currentPerson,
      required this.expected,
      required this.fullPronoun,
      required this.language})
      : super(key: key);

  final int currentPerson;
  final String expected;
  final String fullPronoun;
  final String language;

  @override
  State<TextChecker> createState() => TextCheckerState();
}

/// The state consists of the Icon (and its color), the comparison status and
/// the word that is actually typed.
class TextCheckerState extends State<TextChecker> {
  late Widget statusIcon = _updatedIcon();
  ComparisonStatus _comparisonStatus = ComparisonStatus.notStarted;
  String actual = '';

  final TextEditingController _actualText = TextEditingController(text: '');

  @override
  void initState() {
    super.initState();
    _actualText.text = '';
  }

  @override
  Widget build(BuildContext context) {
    return Card(
      elevation: 20,
      child: Row(
        children: [
          PronounField(
            targetLanguage: widget.language,
            expectedIndicative: widget.expected,
            fullPronoun: widget.fullPronoun,
          ),
          Expanded(child: _actualTextField(context, widget.currentPerson)),
          _updatedIcon(),
        ],
      ),
    );
  }

  Widget _actualTextField(BuildContext context, int person) {
    return SizedBox(
      height: 40,
      child: TextField(
        key: Key('actualVerb_$person'),
        obscureText: false,
        autofocus: true,
        decoration: const InputDecoration(
          border: OutlineInputBorder(),
        ),
        readOnly: false,
        controller: _actualText,
        onChanged: (input) {
          actual = _actualText.text;
          _validateInput(context, widget.currentPerson, widget.expected);
          setState(() {
            statusIcon = _updatedIcon();
          });
        },
      ),
    );
  }

  void _validateInput(BuildContext context, int person, String expected) {
    if (actual.isEmpty) {
      _comparisonStatus = ComparisonStatus.notStarted;
    } else if (actual.length > expected.length) {
      _comparisonStatus = ComparisonStatus.moreThanOneWrong;
    } else if (actual.length == expected.length) {
      if (actual == expected) {
        _comparisonStatus = ComparisonStatus.completelyCorrect;
      }
    } else if (actual.length < expected.length) {
      if (expected.substring(0, actual.length) == actual) {
        _comparisonStatus = ComparisonStatus.stillCorrect;
      } else {
        _comparisonStatus = ComparisonStatus.firstWrong;
      }
    }
  }

  Widget _updatedIcon() {
    return Padding(
      padding: const EdgeInsets.all(8.0),
      child: Icon(
        Icons.assignment_turned_in_outlined,
        key: const Key('comparisonStatusIcon'),
        color: _updateIconColor(),
      ),
    );
  }

  Color _updateIconColor() {
    switch (_comparisonStatus) {
      case ComparisonStatus.notStarted:
        return Colors.grey;
      case ComparisonStatus.stillCorrect:
        return Colors.orange;
      case ComparisonStatus.completelyCorrect:
        return Colors.green;
      case ComparisonStatus.firstWrong:
        return Colors.red;
      case ComparisonStatus.moreThanOneWrong:
        return Colors.red;
      default:
        return Colors.grey;
    }
  }
}

/// Status of comparison between an expected indicative and what is actually
/// typed.
enum ComparisonStatus {
  notStarted,
  firstWrong,
  moreThanOneWrong,
  stillCorrect,
  completelyCorrect
}
